<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class UA_WEB_NOTIFICATION_INTEGRATION {

	public function __construct() {
		add_action( 'init', array( $this, 'add_rewrite_rule' ) );
		add_filter( 'query_vars', array( $this, 'add_query_vars' ) );
		add_action( 'wp', array( $this, 'maybe_render' ) );
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_script' ) );
		add_action( 'wp_footer', array( $this, 'include_ua_script' ) );
	}

	/**
     * Add rewrite rules to access worker js at domain root
     */
	public function add_rewrite_rule() {
		if( ! $this->can_integrate() ) {
			return;
		}
		add_rewrite_rule( '^push-worker\.js$', 'index.php?ua_wn_worker=1', 'top' );

		if( $this->add_secure_bridge() ) {
			add_rewrite_rule( '^secure-bridge\.html$', 'index.php?ua_wn_secure_bridge=1', 'top' );
		}
	}

	/**
     * Add query vars which are used in rewrite rules
     *
     * @param $vars
     * @return array
     */
    public function add_query_vars( $vars ) {
		if( $this->can_integrate() ) {
			$vars[] = 'ua_wn_worker';
			if( $this->add_secure_bridge() ) {
			    $vars[] = 'ua_wn_secure_bridge';
			}
		}

		return $vars;
	}

	/**
     * Render worker js and secure bridge
     */
	public function maybe_render() {
		if( ! $this->can_integrate() ) {
			return;
		}

		// worker js
		if ( get_query_var( 'ua_wn_worker' ) ) {
			$ua_web_notify = UA_WEB_NOTIFICATION::instance();
			$default_icon = $ua_web_notify->settings->get_default_image_url();
			$default_title = $ua_web_notify->settings->get_default_title();
			$default_url = $ua_web_notify->settings->get_default_action_url();
			$app_key = $ua_web_notify->settings->get_app_key();
			$token = $ua_web_notify->settings->get_token();
			$vapid_pub_key = $ua_web_notify->settings->get_vapid_public_key();

			if( $this->add_secure_bridge() ) {
			    $url = trailingslashit( home_url( '', 'https' ) ) . 'secure-bridge.html';
                $secure_iframe = ",
        secureIframeUrl: '{$url}'";
			} else {
				$secure_iframe = '';
			}

			header( 'Content-Type: application/javascript' );
			?>
importScripts('https://web-sdk.urbanairship.com/notify/v1/ua-sdk.min.js');
uaSetup.worker(self, {
	defaultIcon: '<?php echo esc_js( $default_icon ) ?>',
	defaultTitle: '<?php echo esc_js( $default_title ) ?>',
	defaultActionURL: '<?php echo esc_js( $default_url ) ?>',
	appKey: '<?php echo esc_js( $app_key ) ?>',
	token: '<?php echo esc_js( $token ) ?>',
	vapidPublicKey: '<?php echo esc_js( $vapid_pub_key ) ?>'<?php echo $secure_iframe ?>
});
<?php
			exit;

        // secure bridge
		} elseif( $this->add_secure_bridge() && get_query_var( 'ua_wn_secure_bridge' ) ) {
			$ua_web_notify = UA_WEB_NOTIFICATION::instance();
			$app_key = $ua_web_notify->settings->get_app_key();

			header( 'Content-type: text/html' );
			?>
<!DOCTYPE html>
<html><head></head><body>
    <script type="text/javascript" src="https://web-sdk.urbanairship.com/notify/v1/ua-sdk.min.js"></script>
    <script type="text/javascript">uaSetup.secureBridge({
        appKey: '<?php echo esc_js( $app_key ) ?>'
    })</script>
</body></html>
<?php
			exit;
		}
	}

	/**
     * Enqueue js script
     */
	public function enqueue_script() {
        if( ! $this->can_integrate() ) {
			return;
		}

		$ua_web_notify = UA_WEB_NOTIFICATION::instance();
        $prompt_settings = array();
        if( is_a( $ua_web_notify->settings, 'UA_WEB_NOTIFICATION_SETTINGS' ) ) {
            $prompt_settings = $ua_web_notify->settings->get_prompt_settings();
        }

        $settings_data = array(
            'prompt' => array(
                'enabled' => empty( $prompt_settings['prompt'] ) ? false : (bool) $prompt_settings['prompt'],
                'prompt_views' => empty( $prompt_settings['prompt_views'] ) ? 1 : intval( $prompt_settings['prompt_views'] ),
                'prompt_again_views' => ! isset( $prompt_settings['prompt_again_views'] ) ? 1 : intval( $prompt_settings['prompt_again_views'] ),
            ),
        );

		wp_enqueue_script(
            'ua-wn',
            UA_WEB_NOTIFICATION_URL . 'assets/js/notification.js',
            array( 'jquery' ),
            UA_WEB_NOTIFICATION_VERSION
		);

		wp_localize_script(
            'ua-wn',
            'uaWnSettings',
            $settings_data
		);
	}

	/**
     * Include UA script on all the pages in footer
     */
	public function include_ua_script() {
	    if( ! $this->can_integrate() ) {
			return;
		}

		$ua_web_notify = UA_WEB_NOTIFICATION::instance();
        $app_key = $ua_web_notify->settings->get_app_key();
        $token = $ua_web_notify->settings->get_token();
        $vapid_pub_key = $ua_web_notify->settings->get_vapid_public_key();

        if( $this->add_secure_bridge() ) {
            $url = trailingslashit( home_url( '', 'https' ) ) . 'secure-bridge.html';
            $secure_iframe = ",
        secureIframeUrl: '{$url}'";
        } else {
            $secure_iframe = '';
        }
	    ?>
<script type="text/javascript">
!function(n,t,c,e,u){function r(n){try{f=n(u)}catch(n){return h=n,void i(p,n)}i(s,f)}function i(n,t){for(var c=0;c<n.length;c++)d(n[c],t);
}function o(n,t){return n&&(f?d(n,f):s.push(n)),t&&(h?d(t,h):p.push(t)),l}function a(n){return o(!1,n)}function d(t,c){
n.setTimeout(function(){t(c)},0)}var f,h,s=[],p=[],l={then:o,catch:a,_setup:r};n[e]=l;var v=t.createElement("script");
v.src=c,v.async=!0,v.id="_uasdk",v.rel=e,t.head.appendChild(v)}(window,document,'https://web-sdk.urbanairship.com/notify/v1/ua-sdk.min.js',
    'UA', {
        appKey: '<?php echo esc_js( $app_key ) ?>',
	    token: '<?php echo esc_js( $token ) ?>',
	    vapidPublicKey: '<?php echo esc_js( $vapid_pub_key ) ?>'<?php echo $secure_iframe ?>
    });
</script>
<?php
	}

	/**
     * Determine if it can integrate or not
     *
     * @return bool
     */
	public function can_integrate() {
		$ua_web_notify = UA_WEB_NOTIFICATION::instance();

		return ( is_a( $ua_web_notify->settings, 'UA_WEB_NOTIFICATION_SETTINGS' ) && $ua_web_notify->settings->is_ua_ac_configured() );
	}

	/**
     * Determine if it should integrate secure bridge or not
     *
     * @return bool
     */
	public function add_secure_bridge() {
	    $add_secure_bridge = false;
	    $ua_web_notify = UA_WEB_NOTIFICATION::instance();
		if ( isset( $ua_web_notify->settings ) && is_a( $ua_web_notify->settings, 'UA_WEB_NOTIFICATION_SETTINGS' ) ) {
		    $add_secure_bridge = ( $ua_web_notify->settings->use_secure_bridge() || apply_filters( 'ua_wn_integrate_html_bridge', false ) );
		}

		return $add_secure_bridge;
	}

}